@@ -40,6 +40,8 @@ class About:
             '../icons/hicolor/scalable/apps/epoptes.svg', 64, 64)
         self.dialog.set_logo(logo)
         self.dialog.set_version(__version__)
+        self.dialog.set_translator_credits(_("translator-credits"))
+        self.dialog.set_artists(["Andrew Wedderburn (application icon)"])
     
     def run(self):
         self.dialog.run()
