@@ -58,22 +58,20 @@ from epoptes.core import structs
 class EpoptesGui(object):
     
     def __init__(self):
-        self.shownCompatibilityWarning = False 
+        self.shownCompatibilityWarning = False
         self.vncserver = None
         self.vncviewer = None
         self.scrWidth = 100
         self.scrHeight = 75
+        self.showRealNames = False
         self.currentScreenshots = dict()
-        self.current_macs = subprocess.Popen(['sh', '-c', 
+        self.current_macs = subprocess.Popen(['sh', '-c',
             """ip -oneline -family inet link show | sed -n '/.*ether[[:space:]]*\\([[:xdigit:]:]*\).*/{s//\\1/;y/abcdef-/ABCDEF:/;p;}'
             echo $LTSP_CLIENT_MAC"""],
             stdout=subprocess.PIPE).communicate()[0].split()
         self.uid = os.getuid()
-        if self.uid != 0:
-            if 'thumbnails_width' in config.user:
-                self.scrWidth = config.user['thumbnails_width']
-            if 'thumbnails_height' in config.user:
-                self.scrHeight = config.user['thumbnails_height']
+        if 'thumbnails_width' in config.user:
+            self.scrWidth = config.user['thumbnails_width']
         self.offline = gtk.gdk.pixbuf_new_from_file('images/offline.svg')
         self.thin = gtk.gdk.pixbuf_new_from_file('images/thin.svg')
         self.fat = gtk.gdk.pixbuf_new_from_file('images/fat.svg')
@@ -85,9 +83,6 @@ class EpoptesGui(object):
         self.wTree.add_from_file('epoptes.ui')
         
         self.get = lambda obj: self.wTree.get_object(obj)
-        self.showRealNames = False
-        if config.settings.has_option('GUI', 'showRealNames'):
-            self.get('mcShowRealNames').set_active(config.settings.getboolean('GUI', 'showRealNames'))
         
         # Connect glade handlers with the callback functions
         self.wTree.connect_signals(self)
@@ -104,9 +99,6 @@ class EpoptesGui(object):
         self.gtree.set_model(self.gstore)
         self.gtree.get_selection().connect("changed", self.on_group_selection_changed)
         
-        # Enable this when the scrollbar for the screenshot size is available
-        #self.get('iconsSizeAdjustment').set_value(self.scrWidth)
-
         self.mainwin = self.get('mainwindow')
         
         self.cstore = gtk.ListStore(str, gtk.gdk.Pixbuf, object, str)
@@ -129,6 +121,9 @@ class EpoptesGui(object):
         self.default_group.ref = gtk.TreeRowReference(self.gstore, self.gstore.get_path(default_iter))
         self.gtree.get_selection().select_path(self.default_group.ref.get_path())
         
+        self.get('iconsSizeAdjustment').set_value(self.scrWidth)
+        self.reload_imagetypes()
+        
         saved_clients, groups = config.read_groups(os.path.expanduser('~/.config/epoptes/groups.json'))
         for grp in groups:
             self.gstore.append([grp.name, grp, True])
@@ -142,22 +137,42 @@ class EpoptesGui(object):
                 self.get(config.settings.get('GUI', 'label')).set_active(True)
             except:
                 pass
+        if config.settings.has_option('GUI', 'showRealNames'):
+            self.get('mcShowRealNames').set_active(config.settings.getboolean('GUI', 'showRealNames'))
         
 
     #################################################################
     #                       Callback functions                      #
     #################################################################
+    def on_gtree_drag_motion(self, widget, context, x, y, etime):
+        drag_info = widget.get_dest_row_at_pos(x, y)
+        # Don't allow dropping in the empty space of the treeview,
+        # or inside the 'Detected' group, or inside the currently selected group
+        selected_path = self.gstore.get_path(self.getSelectedGroup()[0])
+        if (not drag_info or drag_info[0] == self.default_group.ref.get_path() or
+            drag_info[0] == selected_path):
+            widget.set_drag_dest_row(None, gtk.TREE_VIEW_DROP_AFTER)
+        else:
+            path, pos = drag_info
+            # Don't allow dropping between the groups treeview rows
+            if pos == gtk.TREE_VIEW_DROP_BEFORE:
+                widget.set_drag_dest_row(path, gtk.TREE_VIEW_DROP_INTO_OR_BEFORE)
+            elif pos == gtk.TREE_VIEW_DROP_AFTER:
+                widget.set_drag_dest_row(path, gtk.TREE_VIEW_DROP_INTO_OR_AFTER)
+        
+        context.drag_status(context.suggested_action, etime)
+        return True
+    
     def on_gtree_drag_drop(self, wid, context, x, y, time):
         dest = self.gtree.get_dest_row_at_pos(x, y)
         if dest is not None:
             path, pos = dest
-            if pos in (gtk.TREE_VIEW_DROP_INTO_OR_BEFORE, gtk.TREE_VIEW_DROP_INTO_OR_AFTER):
-                group = self.gstore[path][G_INSTANCE]
-                if not group is self.default_group:
-                    for cln in self.getSelectedClients():
-                        cln = cln[C_INSTANCE]
-                        if not group.has_client(cln):
-                            group.add_client(cln)
+            group = self.gstore[path][G_INSTANCE]
+            if not group is self.default_group:
+                for cln in self.getSelectedClients():
+                    cln = cln[C_INSTANCE]
+                    if not group.has_client(cln):
+                        group.add_client(cln)
     
         context.finish(True, False, time)
         return True
@@ -174,21 +189,16 @@ class EpoptesGui(object):
         settings = config.settings
         if not settings.has_section('GUI'):
             settings.add_section('GUI')        
-        if self.get('miClientsViewHostUser').get_active():
-            label = 0
-        elif self.get('miClientsViewHost').get_active():
-            label = 1
-        elif self.get('miClientsViewUserHost').get_active():
-            label = 2
-        elif self.get('miClientsViewUser').get_active():
-            label = 3
         
         settings.set('GUI', 'selected_group', sel_group)
         settings.set('GUI', 'showRealNames', self.showRealNames)
-        f = open(os.path.expanduser('~/.config/epoptes/settings'), 'w')
-        settings.write(f)
-        f.close()
-        
+        settings.set('GUI', 'thumbnails_width', self.scrWidth)
+        try:
+            f = open(os.path.expanduser('~/.config/epoptes/settings'), 'w')
+            settings.write(f)
+            f.close()
+        except:
+            pass
         if not self.vncserver is None:
             self.vncserver.kill()
         if not self.vncviewer is None:
@@ -226,7 +236,7 @@ class EpoptesGui(object):
             warn=_('Are you sure you want to log off all the users?'))
 
 
-    def reverseConnection(self, widget, path, view_column, cmd):
+    def reverseConnection(self, widget, path, view_column, cmd, *args):
         # Open vncviewer in listen mode
         if self.vncviewer is None:
             self.vncviewerport = self.findUnusedPort()
@@ -234,16 +244,24 @@ class EpoptesGui(object):
             if os.path.isfile('/usr/bin/ssvncviewer'):
                 self.vncviewer = subprocess.Popen(['ssvncviewer',
                     '-multilisten', str(self.vncviewerport-5500)])
-            else:
+            elif os.path.isfile('/usr/bin/xvnc4viewer'):
                 self.vncviewer = subprocess.Popen(['xvnc4viewer',
                     '-listen', str(self.vncviewerport)])
+            else:
+                self.vncviewer = subprocess.Popen(['vncviewer',
+                    '-listen', str(self.vncviewerport-5500)])
 
         # And, tell the clients to connect to the server
-        self.execOnSelectedClients([cmd, self.vncviewerport])
+        self.execOnSelectedClients([cmd, self.vncviewerport] + list(args))
 
 
     def assistUser(self, widget, path=None, view_column=None):
-        self.reverseConnection(widget, path, view_column, 'get_assisted')
+        if config.settings.has_option('GUI', 'grabkbdptr'):
+            grab = config.settings.getboolean('GUI', 'grabkbdptr')
+        if grab:
+            self.reverseConnection(widget, path, view_column, 'get_assisted', 'True')
+        else:
+            self.reverseConnection(widget, path, view_column, 'get_assisted')
 
 
     def monitorUser(self, widget, path=None, view_column=None):
@@ -276,11 +294,12 @@ class EpoptesGui(object):
             self.vncserverport = self.findUnusedPort()
             self.vncserver = subprocess.Popen(['x11vnc', '-noshm', '-nopw',
                 '-quiet', '-viewonly', '-shared', '-forever', '-nolookup',
-                '-24to32', '-rfbport', str(self.vncserverport), '-rfbauth', pwdfile])
+                '-24to32', '-threads', '-rfbport', str(self.vncserverport),
+                '-rfbauth', pwdfile])
         self.execOnSelectedClients(['stop_screensaver'],
             mode=EM_SYSTEM_AND_SESSION)
         self.execOnSelectedClients(["receive_broadcast", self.vncserverport, 
-                                    self.pwd, fullscreen], mode=EM_SYSTEM_OR_SESSION)
+            self.pwd, fullscreen], mode=EM_SYSTEM_OR_SESSION)
 
 
     def broadcastScreen(self, widget):
@@ -389,7 +408,7 @@ class EpoptesGui(object):
         if self.warnDlgPopup(_('Are you sure you want to remove the selected client(s) from group "%s"?') %group.name):
             for client in clients:
                 group.remove_client(client[C_INSTANCE])
-            self.fillIconView(self.getSelectedGroup()[1])
+            self.fillIconView(self.getSelectedGroup()[1], True)
 
 
     def set_move_group_sensitivity(self):
@@ -518,7 +537,7 @@ class EpoptesGui(object):
         if not client is None:
             for row in self.cstore:
                 if row[C_INSTANCE] is client: 
-                    self.fillIconView(self.getSelectedGroup()[1])
+                    self.fillIconView(self.getSelectedGroup()[1], True)
                     break
 
 
@@ -564,8 +583,10 @@ class EpoptesGui(object):
             self.cstore.append([self.calculateLabel(client), self.imagetypes[client.type], client, ''])
 
 
-    def fillIconView(self, group):
+    def fillIconView(self, group, keep_selection=False):
         """Fill the clients iconview from a Group class instance."""
+        if keep_selection:
+            selection = [row[C_INSTANCE] for row in self.getSelectedClients()]
         self.cstore.clear()
         if self.isDefaultGroupSelected():
             clients_list = [client for client in structs.clients if client.type != 'offline']
@@ -574,6 +595,11 @@ class EpoptesGui(object):
         # Add the new clients to the iconview
         for client in clients_list:
             self.addToIconView(client)
+        if keep_selection:
+            for row in self.cstore:
+                if row[C_INSTANCE] in selection:
+                    self.cview.select_path(row.path)
+                    selection.remove(row[C_INSTANCE])
 
 
     def isDefaultGroupSelected(self):
@@ -656,7 +682,7 @@ which is incompatible with the current epoptes version.\
                 loginNotify(user, host)
         
         if sel_group.has_client(client) or self.isDefaultGroupSelected():
-            self.fillIconView(sel_group)
+            self.fillIconView(sel_group, True)
 
 
     def setLabel(self, row):
@@ -694,13 +720,8 @@ which is incompatible with the current epoptes version.\
 
     def getShowRealNames(self):
         return self.get('')
-
-
-    def getAllScreenshots(self):
-        # TODO: Ask for screenshots for every client (Look diff at Rev:326)
-        pass
-
-
+    
+    
     def screenshotTimeout(self, handle):
         print "Screenshot for client %s timed out. Requesting a new one..." % handle
         self.askScreenshot(handle)
@@ -803,30 +824,90 @@ which is incompatible with the current epoptes version.\
             user = "epoptes_user." # The dot is converted to a random digit
         self.openLink("http://webchat.freenode.net/?nick=" + user + 
             "&channels=ltsp&prompt=1")
-
-
-    ## FIXME: We don't use this (we want to). there was a problem with twisted :-\
-    def iconsSizeScaleChanged(self, widget):
+    
+    
+    def iconsSizeScale_button_event(self, widget, event):
+        """A hack to make the left click work like middle click
+        in the scale (for jump-to functionality). Right click resets
+        the thumbnail size.
+        """
+        if event.button == 1:
+            event.button = 2
+        elif event.button == 3:
+            self.iconsSizeScaleChanged(None, 120) # Reset the thumbnail size
+        return False
+    
+    
+    def reload_imagetypes(self):
+        """Improve the quality of previously resized svg icons,
+        by reloading them.
+        """
+        old_pixbufs = self.imagetypes.values()
+        loadSVG = lambda path: gtk.gdk.pixbuf_new_from_file_at_size(path, 
+                                                  self.scrWidth, self.scrHeight)
+        self.imagetypes = {
+            'offline' : loadSVG('images/offline.svg'),
+            'thin' : loadSVG('images/thin.svg'),
+            'fat' : loadSVG('images/fat.svg'),
+            'standalone' : loadSVG('images/standalone.svg')
+        }
+        
+        rows = [row for row in self.cstore if row[C_PIXBUF] in old_pixbufs]
+        for row in rows:
+            row[C_PIXBUF] = self.imagetypes[row[C_INSTANCE].type]
+    
+    
+    def on_iconsSizeScale_button_release_event(self, widget=None, event=None):
+        # Here we want to resize the SVG icons from imagetypes at a better
+        # quality than this of the quick pixbuf scale, since we assume that
+        # the user has decided the desired zoom level.
+        self.reload_imagetypes()
+    
+    
+    def iconsSizeScaleChanged(self, widget=None, width=None):
         adj = self.get('iconsSizeAdjustment')
-        self.scrWidth = int(adj.get_value())
+        if width:
+            adj.set_value(width)
+        else:
+            width = adj.get_value()
+        self.scrWidth = int(width)
         self.scrHeight = int(3 * self.scrWidth / 4) # Κeep the 4:3 aspect ratio
-        self.getAllScreenshots()
-
-
+        
+        # Fast scale all the thumbnails to make the change quickly visible
+        old_pixbufs = self.imagetypes.values()
+        for row in self.cstore:
+            if row[C_PIXBUF] in old_pixbufs:
+                ctype = row[C_INSTANCE].type
+                cur_w = self.imagetypes[ctype].get_width()
+                cur_h = self.imagetypes[ctype].get_height()
+                if not (cur_w == self.scrWidth and cur_h == self.scrHeight):
+                    new_thumb = row[C_PIXBUF].scale_simple(self.scrWidth, self.scrHeight, gtk.gdk.INTERP_NEAREST)
+                    self.imagetypes[ctype] = new_thumb
+                row[C_PIXBUF] = self.imagetypes[ctype]
+            else:
+                new_thumb = row[C_PIXBUF].scale_simple(self.scrWidth, self.scrHeight, gtk.gdk.INTERP_NEAREST)
+                row[C_PIXBUF] = new_thumb
+        
+        # Hack to remove the extra padding that remains after a 'zoom out'
+        self.cview.set_resize_mode(gtk.RESIZE_IMMEDIATE)
+        self.cview.get_cells()[1].set_fixed_size(-1, -1)
+        self.cview.check_resize()
+    
+    
     def scrIncreaseSize(self, widget):
         # Increase the size of screenshots by 2 pixels in width
         adj = self.get('iconsSizeAdjustment')
-        adj.set_value(adj.get_value() + 2)
+        adj.set_value(adj.get_value() + 15)
+        self.reload_imagetypes()
 
 
     def scrDecreaseSize(self, widget):
         # Decrease the size of screenshots by 2 pixels in width
         adj = self.get('iconsSizeAdjustment')
-        adj.set_value(adj.get_value() - 2)
+        adj.set_value(adj.get_value() - 15)
+        self.reload_imagetypes()
+    
     
-    ## END_FIXME
-
-
     def contextMenuPopup(self, widget, event):
         clicked = widget.get_path_at_pos(int(event.x), int(event.y))
 
@@ -863,10 +944,15 @@ which is incompatible with the current epoptes version.\
         self.get('miClientProperties').set_sensitive(sensitive)
         self.get('tb_client_properties').set_sensitive(sensitive)
         
-        if len(selected) >= 1 and not self.isDefaultGroupSelected():
+        if len(selected) > 0 and not self.isDefaultGroupSelected():
             self.get('miRemoveFromGroup').set_sensitive(True)
         else:
             self.get('miRemoveFromGroup').set_sensitive(False)
+        
+        if len(selected) > 1:
+            self.get('statusbar_label').set_text(_('%d clients selected' % len(selected)))
+        else:
+            self.get('statusbar_label').set_text('')
 
 
     def execOnClients(self, command, clients=[], reply=None,
